import threading
from os import listdir

from .default_settings import DOWNLOAD_SOURCE, IFACE
from .default_settings import MODEM_CONSOLE_PATH, MEASUREMENT_TYPE
from .exceptions import LtemetrError, ConfigError
from .logging import get_logger
from .lteinfo import get_operator
from .measurement import Measurement
from .measurement import MeasurementTypes as MT
from .netmetr import run_netmetr
from .uci import UCIConfig

UCI_CONF_NAME = "ltemetr.settings"

logger = get_logger()


def upload_measurement(measurement):
    # TODO do something!
    pass


class StoppableThread(threading.Thread):
    def __init__(self, args):
        super(StoppableThread, self).__init__(args=args)
        self.runner = args[0]
        self._stop_event = threading.Event()

    def stop(self):
        self._stop_event.set()

    def join(self, *args, **kwargs):
        self.stop()
        super(StoppableThread, self).join(*args, **kwargs)

    def run(self):
        self.runner.errors = []
        try:
            self._run_measurement()
        except LtemetrError as e:
            logger.error("Measurement terminated due to: %s", e)
            self.runner.errors = ["Measurement terminated due to: {}".format(e)]
            self.runner.thread = None

    def _run_measurement(self):
        if self.runner.measurement_type == MT.TRAIN:
            while not self._stop_event.is_set():
                measurement = Measurement(
                    download_src=self.runner.download_src,
                    iface=self.runner.iface,
                    modem_console_path=self.runner.modem_console_path,
                    stop_event=self._stop_event.is_set
                )
                self.runner.measurement = measurement
                measurement.measure()
                if not self._stop_event.is_set():
                    upload_measurement(measurement)
                    run_netmetr(self.runner.modem_console_path)
        elif self.runner.measurement_type == MT.CAR:
            measurement = Measurement(
                download_src=self.runner.download_src,
                iface=self.runner.iface,
                modem_console_path=self.runner.modem_console_path,
                stop_event=self._stop_event.is_set
            )
            self.runner.measurement = measurement
            while not self._stop_event.is_set():
                measurement.measure()
        else:
            raise LtemetrError("Unknown measurement type ({})"
                               "".format(self.runner.measurement_type))


class Runner():
    thread = None
    measurement = Measurement()
    errors = []
    uci = UCIConfig(UCI_CONF_NAME)
    try:
        measurement_type = uci.get("measurement_type")
    except ConfigError as e:
        logger.error("Measurement type not loaded from uci (%s), falling back to "
                     "%s", e, MEASUREMENT_TYPE)
        measurement_type = MEASUREMENT_TYPE
    try:
        iface = uci.get("iface")
    except ConfigError as e:
        logger.error("Interface not loaded from uci (%s), falling back to "
                     "%s", e, IFACE)
        iface = IFACE
    try:
        download_src = uci.get("download_source")
    except ConfigError as e:
        logger.error("Source address to download not loaded from uci (%s), "
                     "falling back to %s", e, DOWNLOAD_SOURCE)
        download_src = DOWNLOAD_SOURCE
    try:
        modem_console_path = uci.get("modem_console_path")
    except ConfigError as e:
        logger.error("Modem console path not loaded from uci (%s), "
                     "falling back to %s", e, MODEM_CONSOLE_PATH)
        modem_console_path = MODEM_CONSOLE_PATH
    try:
        upload_path = uci.get("upload_path")
        upload_secret = uci.get("upload_secret")
    except ConfigError as e:
        logger.error("Upload credintials not loaded from uci (%s), "
                     "upload will be skipped", e)
        upload_path = None

    try:
        operator = get_operator(modem_console_path)
    except LtemetrError as e:
        operator = "Unknown"
        logger.error("Operator get failed (%s)", e)

    def start(self):
        if self.thread is None:
            self.thread = StoppableThread(args=(self,))
            self.thread.start()
            running = self.thread.is_alive()
        else:
            running = self.thread.is_alive()
        return running

    def stop(self):
        if self.thread is not None:
            self.thread.join()
            running = self.thread.is_alive()
            self.thread = None
        else:
            running = False
        return running

    def status(self):
        if self.thread is not None:
            running = self.thread.is_alive()
        else:
            running = False
        return running

    def export(self):
        return "\n".join(self.measurement.log)

    def get_latest(self):
        if not self.measurement.last_time:
            return {}
        latest = self.measurement.last_lte
        latest.update({
            "time": "{:%d.%m.%Y %H:%M:%S}".format(self.measurement.last_time),
            "IP Thrpt": self.measurement.last_kbps,
        })
        return latest

    def get_settings(self):
        return {
            "measurement_type": self.measurement_type,
            "upload_path": self.upload_path,
            "iface": self.iface,
            "download_src": self.download_src,
            "modem_console_path": self.modem_console_path,
            "ifaces": listdir("/sys/class/net/")
        }

    def save_settings(self, settings):
        logger.debug("Received new settings: {}".format(settings))

        if "iface" in settings:
            if self.iface != settings["iface"]:
                ifaces = listdir("/sys/class/net/")
                if settings["iface"] in ifaces:
                    logger.debug("Setting iface to {}".format(settings["iface"]))
                    self.iface = settings["iface"]
                    try:
                        self.uci.set("iface", self.iface)
                    except ConfigError as e:
                        logger.error("Saving iface failed ({})".format(e))
                        raise LtemetrError("Saving iface failed (%)", e) from e
                else:
                    raise Exception("interface '" + settings["iface"] +
                                    "' not found in this system")

        if "download_src" in settings:
            if self.download_src != settings["download_src"]:
                self.download_src = settings["download_src"]
                logger.debug("Setting download source to ({})".format(self.download_src))
                try:
                    self.uci.set("download_source", self.download_src)
                except ConfigError as e:
                    logger.error("Saving download source failed ({})".format(e))
                    raise LtemetrError("Saving download source failed (%)", e) from e

        if "modem_console_path" in settings:
            if self.modem_console_path != settings["modem_console_path"]:
                self.modem_console_path = settings["modem_console_path"]
                logger.debug("Setting modem console path to ({})".format(self.modem_console_path))
                try:
                    self.uci.set("modem_console_path", self.modem_console_path)
                except ConfigError as e:
                    logger.error("Saving modem console path failed ({})".format(e))
                    raise LtemetrError("Saving modem console path failed (%)", e) from e
